#! /usr/bin/env python3
from sqlalchemy import create_engine
from schmeckels.models import Tag, Transaction
from schmeckels.helper import format_amount, get_session
import click
from math import floor
from datetime import datetime, date
from sqlalchemy import func, and_
from dateutil.relativedelta import relativedelta


@click.command(name="stats")
def command():
    outgoing = {}
    incoming = {}
    session = get_session()

    tags = session.query(Tag).filter_by(reporting=True).all()

    # Get start and end of timerange
    year = 2021
    start_date = date(year=year,month=1,day=1)
    end_date = date(year=year,month=12,day=31)

    for tag in tags:
        tag_sum = (
            session.query(func.sum(Transaction.amount))
            .filter(and_(Transaction.date >= start_date, Transaction.date <= end_date))
            .filter(Transaction.tags.any(id=tag.id))
            .first()[0]
        )

        if tag_sum < 0:
            outgoing[tag] = {"sum": tag_sum}
        else:
            incoming[tag] = {"sum": tag_sum}

    sum_outgoing = sum(x["sum"] for x in outgoing.values())
    sum_incoming = sum(x["sum"] for x in incoming.values())

    outgoing = {k: format_amount(v) for k, v in sorted(outgoing.items(), key=lambda item: float(item[1]["sum"]))}
    incoming = {k: format_amount(v) for k, v in sorted(incoming.items(), key=lambda item: float(item[1]["sum"]))}

    print(f"============== Report for {year} =============\n")
    for name, data in outgoing.items():
        print(" {:<30} {:>10} €".format(name.name, data['sum']))
    print("-"*44)
    print(" {:<30} {:>10} €".format("AUSGABEN", format_amount(sum_outgoing)))

    print("\n")

    for name, data in incoming.items():
        print(" {:<30} {:>10} €".format(name.name, data['sum']))
    print("-"*44)
    print(" {:<30} {:>10} €".format("EINNAHMEN", format_amount(sum_incoming)))

    print("\n")
    print("="*44)
    print(" {:<30} {:>10} €".format("GEWINN", format_amount(sum_incoming+sum_outgoing)))
